from __future__ import division
import math

import numpy as np
from numpy import pi

import numdifftools.fornberg as ndf

@np.vectorize
def CxDerivative(f, z0, n=1, contour=None, absIntegrationTol=1e-10, verbose=False):
    r"""
    Compute the derivaive of an analytic function using Cauchy's
    Integral Formula for Derivatives.

    .. math::

        f^{(n)}(z_0) = \frac{n!}{2\pi i} \oint_C \frac{f(z)}{(z-z_0)^{n+1}} dz

    Parameters
    ----------
    f : function
        Function of a single variable f(x).
    z0 : complex
        Point to evaluate the derivative at.
    n : int
        The order of the derivative to evaluate.
    contour : :class:`Contour <cxroots.Contour.Contour>`, optional
        The contour, C, in the complex plane which encloses the point z0.
        By default the contour is the circle |z-z_0|=1e-3.
    absIntegrationTol : float, optional
        The absolute tolerance required of the integration routine.
    verbose : bool, optional
        If True runtime information will be printed.  False be default.

    Returns
    -------
    f^{(n)}(z0) : complex
        The nth derivative of f evaluated at z0
    """
    if contour is None:
        from .contours.Circle import Circle
        C = lambda z0: Circle(z0, 1e-3)
    else:
        C = lambda z0: contour

    integrand = lambda z: f(z)/(z-z0)**(n+1)
    integral = C(z0).integrate(integrand, absTol=absIntegrationTol, verbose=verbose)
    return integral * math.factorial(n)/(2j*pi)


def find_multiplicity(root, f, df=None, rootErrTol=1e-10, verbose=False):
    """
    Find the multiplicity of a given root of f by computing the
    derivatives of f, f^{(1)}, f^{(2)}, ... until
    |f^{(n)}(root)|>rootErrTol.  The multiplicity of the root is then
    equal to n.  The derivative is calculated with `numdifftools <http://numdifftools.readthedocs.io/en/latest/api/numdifftools.html#numdifftools.fornberg.derivative>`_
    which employs a method due to Fornberg.

    Parameters
    ----------
    root : complex
        A root of f, f(root)=0.
    f : function
        An analytic function of a single complex variable such that
        f(root)=0.
    df : function, optional
        The first derivative of f.  If not known then df=None.
    contour : Contour, optional
        The integration contour used to evaluate the derivatives.
    rootErrTol : float, optional
        It will be assumed that f(z)=0 if numerically |f(z)|<rootErrTol.
    verbose : bool, optional
        If True runtime information will be printed.  False be default.

    Returns
    -------
    multiplicity : int
        The multiplicity of the given root.
    """
    if abs(f(root)) > rootErrTol:
        raise ValueError("""
            The provided 'root' is not a root of the given function f.
            Specifically, %f = abs(f(root)) > rootErrTol = %f
            """%(abs(f(root)), rootErrTol))

    n = 1
    while True:
        if df is not None:
            if n==1:
                err = abs(df(root))
            else:
                # ndf.derivative returns an array [f, f', f'', ...]
                err = abs(ndf.derivative(df, root, n-1)[n-1])
        else:
            err = abs(ndf.derivative(f, root, n)[n])

        if verbose:
            print('n', n, '|df^(n)|', err)

        if err > rootErrTol:
            break

        n += 1

    return n




from __future__ import division
import functools

import numpy as np
import scipy.linalg

from .CountRoots import prod

def approximate_roots(C, N, f, df=None, absTol=1e-12, relTol=1e-12,
    errStop=1e-10, divMin=3, divMax=15, m=2, rootTol=1e-8,
    intMethod='quad', callback=None, verbose=False):
    """
    Approximate the roots and multiplcities of the function f within the
    contour C using the method of [KB]_.  The multiplicites are computed
    using eq. (21) in [SLV]_.

    Parameters
    ----------
    C : :class:`~<cxroots.Contour.Contour>`
        The contour which encloses the roots of f the user wishes to find.
    N : int
        The number of roots (counting multiplicties) of f within C.
        This is the result of calling :meth:`~cxroots.Contour.Contour.count_roots`.
    f : function
        The function for which the roots are sought.  Must be a function
        of a single complex variable, z, which is analytic within C and
        has no poles or roots on the C.
    df : function, optional
        A function of a single complex variable which is the derivative
        of the function f(z). If df is not given then it will be
        approximated with a finite difference formula.
    absTol : float, optional
        Absolute error tolerance for integration.
    relTol : float, optional
        Relative error tolerance for integration.
    errStop : float, optional
        The number of distinct roots within a contour, n, is determined
        by checking if all the elements of a list of contour integrals
        involving formal orthogonal polynomials are sufficently close to
        zero, ie. that the absolute value of each element is < errStop.
        If errStop is too large/small then n may be smaller/larger than
        it actually is.
    divMin : int, optional
        If the Romberg integration method is used then divMin is the
        minimum number of divisions before the Romberg integration
        routine is allowed to exit.
    divMax : int, optional
        If the Romberg integration method is used then divMax is the
        maximum number of divisions before the Romberg integration
        routine exits.
    m : int, optional
        Only used if df=None and method='quad'.  The argument order=m is
        passed to numdifftools.Derivative and is the order of the error
        term in the Taylor approximation.  m must be even.
    rootTol : float, optional
        If any roots are within rootTol of one another then they will be
        treated as duplicates and removed.  This helps to alleviate the
        problem of errStop being too small.
    intMethod : {'quad', 'romb'}, optional
        If 'quad' then :func:`scipy.integrate.quad` is used to perform
        integration.  If 'romb' then Romberg integraion is performed
        instead.
    callback : function, optional
        Only used if intMethod is 'romb'.  Passed to :func:`~<cxroots.CountRoots.prod>`.
    verbose : bool, optional
        If True certain information regarding the rootfinding process
        will be printed.

    Returns
    -------
    tuple of complex
        The distinct roots of f within the contour C.
    tuple of float
        The corresponding multiplicites of the roots within C.  Should
        be integers but will not be automatically rounded here.

    References
    ----------
    .. [KB] P. Kravanja and M. Van Barel. "Computing the Zeros of
        Anayltic Functions". Springer (2000)
    .. [SLV] E. Strakova, D. Lukas, P. Vodstrcil. "Finding Zeros of
        Analytic Functions and Local Eigenvalue Analysis Using Contour
        Integral Method in Examples". Mathematical Analysis and Numerical
        Mathematics, Vol. 15, 2, (2017)
    """
    if verbose:
        print('Approximating roots in: ' + str(C))
        print('The number of roots, counting multiplcities, within this contour is', N)

    if N == 0:
        return (), ()

    product = functools.partial(prod, C, f, df,
        absTol=absTol, relTol=relTol, divMin=divMin, divMax=divMax,
        m=m, intMethod=intMethod, verbose=verbose, callback=callback)

    s = [N, product(lambda z: z)[0]]    # ordinary moments
    mu = s[1]/N
    phiZeros = [[],[mu]]

    def phi(i):
        if len(phiZeros[i]) == 0:
            return lambda z: np.ones_like(z)
        else:
            coeff = np.poly(phiZeros[i])
            return lambda z: np.polyval(coeff, z)

    # initialize G_{pq} = <phi_p, phi_q>
    G = np.zeros((N,N), dtype=np.complex128)
    G[0,0] = N # = <phi_0, phi_0> = <1,1>

    # initialize G1_{pq} = <phi_p, phi_1 phi_q>
    G1 = np.zeros((N,N), dtype=np.complex128)
    G1[0,0] = 0 # = <phi_0, phi_1 phi_0> = <1, z-mu> = s1-mu*N = 0

    r, t = 1, 0
    while r+t<N:
        ### define FOP of degree r+t+1
        p = r+t
        G[p, 0:p+1] = [product(phi(p), phi(q))[0] for q in range(r+t+1)]
        G[0:p+1, p] = G[p, 0:p+1] # G is symmetric
        if verbose: print('G ', G[:p+1,:p+1])

        G1[p, 0:p+1] = [product(phi(p), lambda z: phi(1)(z)*phi(q)(z))[0] for q in range(r+t+1)]
        G1[0:p+1, p] = G1[p, 0:p+1] # G1 is symmetric
        if verbose: print('G1', G1[:p+1,:p+1])

        """
        If any of the zeros of the FOP are outside of the interior
        of the contour then we assume that they are 'arbitary' and
        instead define the FOP as an inner polynomial. [KB]
        """
        polyRoots = scipy.linalg.eig(G1[:p+1,:p+1], G[:p+1,:p+1])[0]+mu
        if np.all([C.contains(z) for z in polyRoots]):
            r, t = r+t+1, 0
            phiZeros.append(polyRoots)

            if verbose: print('Regular poly', r+t, 'roots:', phiZeros[-1])

            # is the number of distinct roots, n=r?
            phiFuncLast = phi(-1)
            for j in range(N-r):
                ip, err = product(lambda z: phiFuncLast(z)*(z-mu)**j, phiFuncLast)

                if verbose: print(j, 'of', N-r, 'err', err, 'abs(ip)', abs(ip))
                if abs(ip) > errStop:
                    # n != r so carry on
                    if verbose: print('n !=', r)
                    break
            else:
                # the for loop did not break
                if verbose: print('n =', r)
                break

        else:
            # define an inner polynomial as phi_{r+t+1} = phi_{t+1} phi_{r}
            t += 1
            phiZeros.append(np.append(phiZeros[t],phiZeros[r]))
            if verbose: print('Inner poly', r+t, 'roots:', phiZeros[-1])

    roots = np.array(phiZeros[-1])

    if verbose:
        print('Computed Roots:')
        print(roots)

    # remove any roots which are not distinct
    rootsToRemove = []
    for i, root in enumerate(roots):
        if len(roots[i+1:]) > 0 and np.any(np.abs(root-roots[i+1:]) < rootTol):
            rootsToRemove.append(i)
    roots = np.delete(roots, rootsToRemove)
    n = len(roots)

    ### compute the multiplicities, eq. (1.19) in [KB]
    # V = np.column_stack([roots**i for i in range(n)])
    # if verbose and n > 2: print('Computing ordinary moments')
    # s += [product(lambda z: z**p)[0] for p in range(2, n)]
    # multiplicities = np.dot(s[:n], np.linalg.inv(V))

    ### compute the multiplicities, eq. (21) in [SLV]
    V = np.array([[phi(j)(root) for root in roots] for j in range(n)])
    multiplicities = np.dot(np.linalg.inv(V), G[:n,0])

    ### The method used in the vandermonde module doesn't seem significantly
    ### better than np.dot(s, np.linalg.inv(V)).  Especially since we know
    ### the result must be an integer anyway.
    # import vandermonde
    # multiplicities_vandermonde = vandermonde.solve_transpose(np.array(roots), np.array(s))

    ### Note that n = rank(H_N) is not used since calculating the
    ### rank of a matrix of floats can be quite unstable
    # s_func = lambda p: prod(C, f, df, lambda z: z**p)[0]
    # HN = np.fromfunction(np.vectorize(lambda p,q: s_func(p+q)), shape=(N,N))
    # print('n?', np.linalg.matrix_rank(HN, tol=1e-10))

    if verbose:
        print('Approximations for roots:\n', roots)
        print('Approximations for multiplicities:\n', multiplicities)

    return tuple(roots), tuple(multiplicities)




from __future__ import division
import warnings

import numpy as np
from numpy import inf, pi
import scipy.integrate
import scipy.misc
import numdifftools.fornberg as ndf
import numdifftools

def prod(C, f, df=None, phi=None, psi=None, absTol=1e-12, relTol=1e-12, divMin=3,
    divMax=15, m=2, intMethod='quad', integerTol=inf, verbose=False, callback=None):
    r"""
    Compute the symmetric bilinear form used in (1.12) of [KB]_.

    .. math::

        <\phi,\psi> = \frac{1}{2\pi i} \oint_C \phi(z) \psi(z) \frac{f'(z)}{f(z)} dz.

    Parameters
    ----------
    C : :class:`Contour <cxroots.Contour.Contour>`
        A contour in the complex plane for.  No roots or poles of f
        should lie on C.
    f : function
        Function of a single variable f(x)
    df : function, optional
        Function of a single variable, df(x), providing the derivative
        of the function f(x) at the point x.  If not provided then df is
        approximated using a finite difference method.
    phi : function, optional
        Function of a single variable phi(x).  If not provided then
        phi(z)=1.
    psi : function, optional
        Function of a single variable psi(x).  If not provided then
        psi(z)=1.
    absTol : float, optional
        Absolute error tolerance for integration.
    relTol : float, optional
        Relative error tolerance for integration.
    divMin : int, optional
        Only used if intMethod='romb'. Minimum number of divisions before
        the Romberg integration routine is allowed to exit.
    divMax : int, optional
        Only used if intMethod='romb'.  The maximum number of divisions
        before the Romberg integration routine of a path exits.
    m : int, optional
        Only used if df=None and intMethod='quad'.  Must be even.  The
        argument order=m is passed to numdifftools.Derivative and is the
        order of the error term in the Taylor approximation.
    intMethod : {'quad', 'romb'}, optional
        If 'quad' then scipy.integrate.quad is used to perform the
        integral.  If 'romb' then Romberg integraion, using
        scipy.integrate.romb, is performed instead.
    integerTol : float, optional
        Only used when intMethod is 'romb'.  The integration routine will
        not exit unless the result is within integerTol of an integer.
        This is useful when computing the number of roots in a contour,
        which must be an integer.  By default integerTol is inf.
    verbose : bool, optional
        If True runtime information will be printed.  False be default.
    callback : function, optional
        Only used when intMethod is 'romb'.  A function that at each
        step in the iteration is passed the current approximation for
        the integral, the estimated error of that approximation and the
        number of iterations.  If the return of callback evaluates to
        True then the integration will end.

    Returns
    -------
    complex
        The value of the integral <phi, psi>.
    float
        An estimate of the error for the integration.

    References
    ----------
    .. [KB] "Computing the zeros of analytic functions" by Peter Kravanja,
        Marc Van Barel, Springer 2000
    """
    if intMethod == 'romb':
        N = 1
        k = 0
        I = []

        while k < divMax and (len(I) < divMin
            or (abs(I[-2] - I[-1]) > absTol and abs(I[-2] - I[-1]) > relTol*abs(I[-1]))
            or (abs(I[-3] - I[-2]) > absTol and abs(I[-3] - I[-2]) > relTol*abs(I[-2]))
            or abs(int(round(I[-1].real)) - I[-1].real) > integerTol
            or abs(I[-1].imag) > integerTol):
            N = 2*N
            t = np.linspace(0,1,N+1)
            k += 1
            dt = t[1]-t[0]

            integrals = []
            for segment in C.segments:
                # compute/retrieve function evaluations
                fVal = segment.trap_values(f,k)

                if df is None:
                    # approximate df/dz with finite difference
                    dfdt = np.gradient(fVal, dt)
                    dfVal = dfdt/segment.dzdt(t)
                else:
                    dfVal = segment.trap_values(df,k)

                segment_integrand = dfVal/fVal*segment.dzdt(t)
                if phi is not None:
                    segment_integrand = segment.trap_values(phi,k)*segment_integrand
                if psi is not None:
                    segment_integrand = segment.trap_values(psi,k)*segment_integrand

                segment_integral = scipy.integrate.romb(segment_integrand, dx=dt, axis=-1)/(2j*pi)
                integrals.append(segment_integral)

            I.append(sum(integrals))

            if verbose:
                if k > 1:
                    print(k, 'I', I[-1], 'err', I[-2] - I[-1])
                else:
                    print(k, 'I', I[-1])

            if callback is not None:
                err = abs(I[-2] - I[-1]) if k > 1 else None
                if callback(I[-1], err, k):
                    break

        return I[-1], abs(I[-2] - I[-1])

    elif intMethod == 'quad':
        if df is None:
            df = numdifftools.Derivative(f, order=m)
            # df = lambda z: scipy.misc.derivative(f, z, dx=1e-8, n=1, order=3)

            ### Too slow
            # ndf.derivative returns an array [f, f', f'', ...]
            # df = np.vectorize(lambda z: ndf.derivative(f, z, n=1)[1])

        I, err = 0, 0
        for segment in C.segments:
            integrand_cache = {}
            def integrand(t):
                if t in integrand_cache.keys():
                    i = integrand_cache[t]
                else:
                    z = segment(t)
                    i = (df(z)/f(z))/(2j*pi) * segment.dzdt(t)
                    if phi is not None:
                        i = phi(z)*i
                    if psi is not None:
                        i = psi(z)*i
                    integrand_cache[t] = i
                return i

            # integrate real part
            integrand_real = lambda t: np.real(integrand(t))
            result_real = scipy.integrate.quad(integrand_real, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
            I_real, abserr_real, infodict_real = result_real[:3]

            # integrate imaginary part
            integrand_imag = lambda t: np.imag(integrand(t))
            result_imag = scipy.integrate.quad(integrand_imag, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
            I_imag, abserr_imag, infodict_imag = result_imag[:3]

            I   += I_real + 1j*I_imag
            err += abserr_real + 1j*abserr_imag

        return I, abs(err)

    else:
        raise ValueError("intMethod must be either 'romb' or 'quad'")

class RootError(RuntimeError):
    pass

def count_roots(C, f, df=None, NIntAbsTol=0.07, integerTol=0.1, divMin=3,
    divMax=15, m=2, intMethod='quad', verbose=False):
    r"""
    For a function of one complex variable, f(z), which is analytic in
    and within the contour C, return the number of zeros (counting
    multiplicities) within the contour, N, using Cauchy's argument
    principle,

    .. math::

        N = \frac{1}{2i\pi} \oint_C \frac{f'(z)}{f(z)} dz.

    If df(z), the derivative of f(z), is provided then the above
    integral is computed directly.  Otherwise the derivative is
    approximated using a finite difference method.

    The number of roots is taken to be the closest integer to the
    computed value of the integral and the result is only accepted
    if the integral is within integerTol of the closest integer.

    Parameters
    ----------
    C : :class:`Contour <cxroots.Contour.Contour>`
        The contour which encloses the roots of f(z) that are to be
        counted.
    f : function
        Function of a single variable f(z).
    df : function, optional
        Function of a single complex variable, df(z), providing the
        derivative of the function f(z) at the point z.  If not
        provided, df will be approximated using a finite difference
        method.
    NIntAbsTol : float, optional
        Required absolute error tolerance for the contour integration.
        Since the Cauchy integral must be an integer it is only
        necessary to distinguish which integer the integral is
        converging towards.  Therefore, NIntAbsTol can be fairly large.
    integerTol : float, optional
        The evaluation of the Cauchy integral will be accepted if its
        value is within integerTol of the closest integer.
    divMin : int, optional
        Only used if intMethod='romb'. Minimum number of divisions
        before the Romberg integration routine is allowed to exit.
    divMax : int, optional
        Only used if intMethod='romb'.  The maximum number of divisions
        before the Romberg integration routine of a path exits.
    m : int, optional
        Only used if df=None and intMethod='quad'.  The argument order=m
        is passed to numdifftools.Derivative and is the order of the
        error term in the Taylor approximation.  m must be even.
    intMethod : {'quad', 'romb'}, optional
        If 'quad' then scipy.integrate.quad is used to perform the
        integral.  If 'romb' then Romberg integraion, using
        scipy.integrate.romb, is performed instead.
    verbose : bool, optional
        If True certain messages regarding the integration will be
        printed.

    Returns
    -------
    int
        The number of zeros of f (counting multiplicities) which lie
        within the contour C.
    """
    if verbose:
        print('Computing number of roots within', C)

    with warnings.catch_warnings():
        # ignore warnings and catch if I is NaN later
        warnings.simplefilter("ignore")
        I, err = prod(C, f, df, absTol=NIntAbsTol, relTol=0, divMin=divMin,
            divMax=divMax, m=m, intMethod=intMethod, verbose=verbose, integerTol=integerTol)

    if intMethod == 'romb':
        C._numberOfDivisionsForN = int(np.log2(len(C.segments[0]._trapValuesCache[f])-1))

    if np.isnan(I):
        raise RootError("""Result of integral is an invalid value.
                           Most likely because of a divide by zero error.""")

    elif abs(int(round(I.real)) - I.real) < integerTol and abs(I.imag) < integerTol:
        # integral is sufficiently close to an integer
        numberOfZeros = int(round(I.real))
        return numberOfZeros

    else:
        raise RootError("The number of enclosed roots has not converged to an integer")




import pytest

from cxroots import AnnulusSector
from cxroots import Rectangle

def test_PolarRect_contains():
    r0=8.938
    r1=9.625
    phi0=6.126
    phi1=6.519

    z = (9-1.04825594683e-18j)
    C = AnnulusSector(0, [r0,r1], [phi0,phi1])

    # import matplotlib.pyplot as plt
    # plt.scatter((9),(0))
    # C.show()

    assert C.contains(z)


def test_rect_contains():
    C = Rectangle([-2355, -1860], [-8810, -8616])

    assert C.contains(-2258-8694j)
    assert not C.contains(-2258-8500j)



from numpydoc.docscrape import FunctionDoc


def remove_para(*paras):
    def wrapper(func):
        doc = FunctionDoc(func)
        for i, p in enumerate(doc['Parameters'][:]):
            if p.name.split(':')[0].rstrip() in paras:
                del doc['Parameters'][i]
        func.__doc__ = doc
        return func
    return wrapper


def update_docstring(**dic):
    def wrapper(func):
        doc = FunctionDoc(func)
        for k, v in dic.items():
            doc[k] = v
        func.__doc__ = doc
        return func
    return wrapper


class NumberOfRootsChanged(Exception):
    pass



import pytest
import numpy as np
from numpy import cos, sin

from cxroots import Rectangle

@pytest.mark.parametrize('useDerivative', [True, False])
def test_count_roots(useDerivative):
    """
    Example from "Locating all the Zeros of an Analytic Function in one Complex Variable"
    M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2

    There should be 424 roots inside this contour
    """
    C  = Rectangle([-20.3,20.7], [-20.3,20.7])
    f  = lambda z: z**50 + z**12 - 5*sin(20*z)*cos(12*z) - 1
    df = lambda z: 50*z**49 + 12*z**11 + 60*sin(12*z)*sin(20*z) - 100*cos(12*z)*cos(20*z)

    if not useDerivative: df = None

    assert C.count_roots(f, df, verbose=True) == 424


@pytest.mark.xfail(reason='Possibly lack of high precision arithmetic')
def test_RingOscillator():
    # problem in section 4.3 of [DSZ]
    from cxroots import Rectangle, findRoots

    def A(z):
        t = 2.
        return np.array([[-0.0166689-2.12e-14*z, 1/60. + 6e-16*exp(-t*z)*z],
                         [0.0166659+exp(-t*z)*(-0.000037485+6e-16*z), -0.0166667-6e-16*z]])

    def dA(z):
        t = 2.
        return np.array([[-2.12e-14*np.ones_like(z), 6e-16*exp(-t*z) - t*6e-16*exp(-t*z)*z],
                         [-t*exp(-t*z)*(-0.000037485+6e-16*z)+exp(-t*z)*6e-16, -6e-16*np.ones_like(z)]])
    
    def f(z):
        AVal = np.rollaxis(A(z),-1,0)
        return np.linalg.det(AVal)

    def df(z):
        AVal  = A(z)
        dAVal = dA(z)
        return dAVal[0,0]*AVal[1,1] + AVal[0,0]*dAVal[1,1] - dAVal[0,1]*AVal[1,0] - AVal[0,1]*dAVal[1,0]


    box = Rectangle([-12,0], [-40,40])
    # roots_fdf = findRoots(box, f, df)
    # roots_f = findRoots(box, f)

    # XXX: No roots are recorded within the initial contour.
    #   Perhaps because the coefficents of z are very small?
    #   Perhaps need higher precision?
    assert box.count_enclosed_roots(f, df) != 0
    assert box.count_enclosed_roots(f) != 0

    # compare with fig 4 of [DSZ]
    # roots_fdf.show()



from __future__ import division

import numpy as np

from .RootFinder import find_roots_gen

def demo_find_roots(originalContour, f, df=None, saveFile=None, automaticAnim=False,
    returnAnim=False, writer=None, **rootsKwargs):
    """
    An animated demonstration of the root finding process using matplotlib.

    Parameters
    ----------
    saveFile : str, optional
        If given then the animation will be saved to disk with filename
        equal to saveFile instead of being shown.
    automaticAnim : bool, optional
        If False (default) then press SPACE to step the animation forward
        If True then the animation will play automatically until all the
        roots have been found.
    returnAnim : bool, optional
        If True then the matplotlib animation object will be returned
        instead of being shown.  Defaults to False.
    writer : str, optional
        Passed to :meth:`matplotlib.animation.FuncAnimation.save`.
    **rootsKwargs
        Additional key word arguments passed to :meth:`~cxroots.Contour.Contour.roots`.
    """
    import matplotlib.pyplot as plt
    from matplotlib import animation
    fig = plt.gcf()
    ax = plt.gca()

    rootFinder = find_roots_gen(originalContour, f, df, **rootsKwargs)

    def init():
        originalContour.plot(linecolor='k', linestyle='--')
        originalContour._sizePlot()

    def update_frame(args):
        roots, multiplicities, boxes, numberOfRemainingRoots = args

        plt.cla() # clear axis
        originalContour.plot(linecolor='k', linestyle='--')
        for box in boxes:
            if not hasattr(box, '_color'):
                cmap = plt.get_cmap('jet')
                box._color = cmap(np.random.random())

            plt.text(box.centralPoint.real, box.centralPoint.imag, box._numberOfRoots)
            box.plot(linecolor=box._color)

        plt.scatter(np.real(roots), np.imag(roots), color='k', marker='x')
        ax.text(0.02, 0.95, 'Zeros remaining: %i'%numberOfRemainingRoots, transform=ax.transAxes)
        originalContour._sizePlot()
        fig.canvas.draw()

    if saveFile:
        automaticAnim = True

    if automaticAnim or returnAnim:
        anim = animation.FuncAnimation(fig, update_frame, init_func=init, frames=rootFinder)
        if returnAnim: return anim

    else:
        def draw_next(event):
            if event.key == ' ':
                update_frame(next(rootFinder))
        fig.canvas.mpl_connect('key_press_event', draw_next)

    if saveFile:
        anim.save(filename=saveFile, fps=1, dpi=200, writer=writer)
        plt.close()
    else:
        plt.show()



import pytest

from cxroots.IterativeMethods import secant
from numpy import pi, cos, sin

def test_secant():
    # example from Table 2.5 of "Numerical Analysis" by Richard L. Burden, J. Douglas Faires
    f  = lambda x: cos(x)-x
    df = lambda x: -sin(x)-1

    iterations = []
    callback = lambda x, dx, y, iteration: iterations.append(x)
    x, err = secant(0.5, pi/4, f, callback=callback)
    iterations.append(x)
    
    correct_iterations = [0.7363841388,
                          0.7390581392,
                          0.7390851493,
                          0.7390851332,
                          0.7390851332]

    assert iterations == pytest.approx(correct_iterations)



import unittest
import numpy as np
from numpy import pi

from cxroots import Circle, Rectangle, AnnulusSector, Annulus
from cxroots.tests.ApproxEqual import roots_approx_equal

class TestRootfindingContours(unittest.TestCase):
    def setUp(self):
        self.roots = roots = [0, -1.234, 1+1j, 1-1j, 2.345]
        self.multiplicities = [1,1,1,1,1]
        self.f  = lambda z: (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[3])*(z-roots[4])
        self.df = lambda z: (z-roots[1])*(z-roots[2])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[2])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[3])
    
        self.Circle = Circle(0,3)
        self.Rectangle = Rectangle([-2,2],[-2,2])
        self.halfAnnulus = AnnulusSector(0, [0.5,3], [-pi/2, pi/2])
        self.Annulus = Annulus(0, [1,2])

    def test_rootfinding_circle_fdf(self):
        roots_approx_equal(self.Circle.roots(self.f, self.df, verbose=True), (self.roots, self.multiplicities), decimal=7)

    def test_rootfinding_circle_f(self):
        roots_approx_equal(self.Circle.roots(self.f, self.df, verbose=True), (self.roots, self.multiplicities), decimal=7)

    def test_rootfinding_rectangle_fdf(self):
        roots_approx_equal(self.Rectangle.roots(self.f, self.df, verbose=True), (self.roots[:-1], self.multiplicities[:-1]), decimal=7)

    def test_rootfinding_rectangle_f(self):
        roots_approx_equal(self.Rectangle.roots(self.f, self.df, verbose=True), (self.roots[:-1], self.multiplicities[:-1]), decimal=7)

    def test_rootfinding_halfAnnulus_fdf(self):
        roots_approx_equal(self.halfAnnulus.roots(self.f, self.df, verbose=True), (self.roots[2:], self.multiplicities[2:]), decimal=7)

    def test_rootfinding_halfAnnulus_f(self):
        roots_approx_equal(self.halfAnnulus.roots(self.f, self.df, verbose=True), (self.roots[2:], self.multiplicities[2:]), decimal=7)

    def test_rootfinding_Annulus_fdf(self):
        roots_approx_equal(self.Annulus.roots(self.f, self.df, verbose=True), (self.roots[1:-1], self.multiplicities[1:-1]), decimal=7)

    def test_rootfinding_Annulus_f(self):
        roots_approx_equal(self.Annulus.roots(self.f, self.df, verbose=True), (self.roots[1:-1], self.multiplicities[1:-1]), decimal=7)

if __name__ == '__main__':
    unittest.main(verbosity=3)



from __future__ import division
from collections import namedtuple

import numpy as np

class RootResult(namedtuple("RootResult", ["roots", "multiplicities"])):
    """
    A class which stores the roots and their multiplicites as attributes
    and provides convienent methods for displaying them.

    Attributes
    ----------
    roots : list
        List of roots
    multiplicities : list
        List of multiplicities where the ith element of the list is the
        multiplicity of the ith element of roots.
    originalContour : Contour
        The contour bounding the region in which the roots were found.
    """
    def __new__(cls, roots, multiplicities, originalContour):
        obj = super(RootResult, cls).__new__(cls, roots, multiplicities)
        obj.originalContour = originalContour
        return obj

    def show(self, saveFile=None):
        """
        Plot the roots and the initial integration contour in the
        complex plane.

        Parameters
        ----------
        saveFile : str, optional
            If provided the plot of the roots will be saved with
            file name saveFile instead of being shown.

        Example
        -------
        .. plot::
            :include-source:

            from cxroots import Circle
            C = Circle(0, 2)
            f = lambda z: z**6 + z**3
            df = lambda z: 6*z**5 + 3*z**2
            r = C.roots(f, df)
            r.show()
        """
        import matplotlib.pyplot as plt
        self.originalContour.plot(linecolor='k', linestyle='--')
        plt.scatter(np.real(self.roots), np.imag(self.roots), color='k', marker='x')

        if saveFile is not None:
            plt.savefig(saveFile)
            plt.close()
        else:
            plt.show()

    def __str__(self):
        roots, multiplicities = np.array(self.roots), np.array(self.multiplicities)

        # reorder roots
        sortargs = np.argsort(roots)
        roots, multiplicities = roots[sortargs], multiplicities[sortargs]

        s =  ' Multiplicity |               Root              '
        s+='\n------------------------------------------------'

        for i, root in np.ndenumerate(roots):
            if root.real < 0:
                s += '\n{: ^14d}| {:.12f} {:+.12f}i'.format(int(multiplicities[i]), root.real, root.imag)
            else:
                s += '\n{: ^14d}|  {:.12f} {:+.12f}i'.format(int(multiplicities[i]), root.real, root.imag)

        return s



import pytest

from cxroots.IterativeMethods import muller
from numpy import pi, cos

def test_muller():
    f  = lambda x: cos(x) - x**2 + 1j*x**3

    iterations = []
    callback = lambda x, dx, y, iteration: iterations.append(x)
    x, err = muller(0.5, pi/4, 0.6, f)

    root = 0.7296100078977741539157356847 + 0.1570923181734581909733787621j
    assert x == pytest.approx(root)




import pytest

from cxroots.IterativeMethods import newton
from numpy import pi, cos, sin

def test_newton():
    # result from keisan online calculator: http://keisan.casio.com/exec/system/1244946907
    f  = lambda x: cos(x)-x
    df = lambda x: -sin(x)-1

    iterations = []
    callback = lambda x, dx, y, iteration: iterations.append(x)
    x, err = newton(pi/4, f, df, callback=callback)
    iterations.append(x)
    
    correct_iterations = [0.73953613351523830094,
                          0.7390851781060101829533,
                          0.7390851332151610866198,
                          0.7390851332151606416553]

    assert iterations == pytest.approx(correct_iterations)




from __future__ import division

import numpy as np
import scipy.integrate
from numpy import exp, pi

class ComplexPath(object):
    """A base class for paths in the complex plane."""
    def __init__(self):
        self._integralCache = {}
        self._trapValuesCache = {}


    def __call__(self, t):
        r"""
        The parameterization of the path in the varaible :math:`t\in[0,1]`.

        Parameters
        ----------
        t : float
            A real number :math:`0\leq t \leq 1`.

        Returns
        -------
        complex
            A point on the path in the complex plane.
        """
        raise NotImplementedError('__call__ must be implemented in a subclass')


    def trap_values(self, f, k, useCache=True):
        """
        Compute or retrieve (if cached) the values of the functions f
        at :math:`2^k+1` points along the contour which are evenly
        spaced with respect to the parameterisation of the contour.

        Parameters
        ----------
        f : function
            A function of a single complex variable.
        k : int
            Defines the number of points along the curve that f is to be
            evaluated at as :math:`2^k+1`.
        useCache : bool, optional
            If True then use, if available, the results of any previous
            calls to this function for the same f and save any new
            results so that they can be reused later.

        Returns
        -------
        :class:`numpy.ndarray`
            The values of f at :math:`2^k+1` points along the contour
            which are evenly spaced with respect to the parameterisation
            of the contour.
        """
        if f in self._trapValuesCache.keys() and useCache:
            vals = self._trapValuesCache[f]
            vals_k = int(np.log2(len(vals)-1))

            if vals_k == k:
                return vals
            elif vals_k > k:
                return vals[::2**(vals_k-k)]
            else:
                t = np.linspace(0, 1, 2**k+1)
                vals = np.empty(2**k+1, dtype=np.complex128)
                vals.fill(np.nan)
                vals[::2**(k-vals_k)] = self._trapValuesCache[f]
                vals[np.isnan(vals)] = f(self(t[np.isnan(vals)]))

                # cache values
                self._trapValuesCache[f] = vals
                return vals

        else:
            t = np.linspace(0, 1, 2**k+1)
            vals = f(self(t))
            if useCache:
                self._trapValuesCache[f] = vals
            return vals


    def plot(self, N=100, linecolor='C0', linestyle='-'):
        """
        Uses matplotlib to plot, but not show, the path as a 2D plot in
        the Complex plane.

        Parameters
        ----------
        N : int, optional
            The number of points to use when plotting the path.
        linecolor : optional
            The colour of the plotted path, passed to the
            :func:`matplotlib.pyplot.plot` function as the keyword
            argument of 'color'.  See the matplotlib tutorial on
            `specifying colours <https://matplotlib.org/users/colors.html#>`_.
        linestyle : str, optional
            The line style of the plotted path, passed to the
            :func:`matplotlib.pyplot.plot` function as the keyword
            argument of 'linestyle'.  The default corresponds to a solid
            line.  See :meth:`matplotlib.lines.Line2D.set_linestyle` for
            other acceptable arguments.
        """
        import matplotlib.pyplot as plt
        t = np.linspace(0,1,N)
        path = self(t)
        plt.plot(path.real, path.imag, color=linecolor, linestyle=linestyle)
        plt.xlabel('Re[$z$]', size=16)
        plt.ylabel('Im[$z$]', size=16)
        plt.gca().set_aspect(1)

        # add arrow to indicate direction of path
        arrow_direction = (self(0.51) - self(0.5))/abs(self(0.51) - self(0.5))
        arrow_extent = 1e-6*arrow_direction
        ymin, ymax = plt.gca().get_ylim()
        xmin, xmax = plt.gca().get_xlim()
        head_length = max(abs(ymax - ymin), abs(xmax - xmin))/40.
        plt.arrow(self(0.5).real, self(0.5).imag,
                  arrow_extent.real, arrow_extent.imag,
                  head_width=head_length*2/3., head_length=head_length,
                  fc=linecolor, ec=linecolor)

    def show(self, saveFile=None, **plotKwargs):
        """
        Shows the path as a 2D plot in the complex plane.  Requires
        Matplotlib.

        Parameters
        ----------
        saveFile : str (optional)
            If given then the plot will be saved to disk with name
            'saveFile'.  If saveFile=None the plot is shown on-screen.
        **plotKwargs
            Other key word arguments are passed to :meth:`~cxroots.Paths.ComplexPath.plot`.
        """
        import matplotlib.pyplot as plt
        self.plot(**plotKwargs)

        if saveFile is not None:
            plt.savefig(saveFile, bbox_inches='tight')
            plt.close()
        else:
            plt.show()

    def integrate(self, f, absTol=0, relTol=1e-12, divMax=15, intMethod='quad', verbose=False):
        """
        Integrate the function f along the path.  The value of the
        integral is cached and will be reused if the method is called
        with same arguments (ignoring verbose).

        Parameters
        ----------
        f : function
            A function of a single complex variable.
        absTol : float, optional
            The absolute tolerance for the integration.
        relTol : float, optional
            The realative tolerance for the integration.
        divMax : int, optional
            If the Romberg integration method is used then divMax is the
            maximum number of divisions before the Romberg integration
            routine of a path exits.
        intMethod : {'quad', 'romb'}, optional
            If 'quad' then :func:`scipy.integrate.quad` is used to
            compute the integral.  If 'romb' then Romberg integraion,
            using :func:`scipy.integrate.romberg`, is used instead.
        verbose : bool, optional
            Passed ass the `show` argument of :func:`scipy.integrate.romberg`.

        Returns
        -------
        complex
            The integral of the function f along the path.

        Notes
        -----
        This function is only used when checking the
        multiplicity of roots.  The bulk of the integration for
        rootfinding is done with :func:`cxroots.CountRoots.prod`.
        """

        args = (f, absTol, relTol, divMax, intMethod)
        if args in self._integralCache.keys():
            integral = self._integralCache[args]

        elif hasattr(self, '_reversePath') and args in self._reversePath._integralCache:
            # if we have already computed the reverse of this path
            integral = -self._reversePath._integralCache[args]

        else:
            integrand = lambda t: f(self(t))*self.dzdt(t)

            if intMethod == 'romb':
                integral = scipy.integrate.romberg(integrand, 0, 1, tol=absTol, rtol=relTol, divmax=divMax, show=verbose)
            elif intMethod == 'quad':
                integrand_real = lambda t: np.real(integrand(t))
                integrand_imag = lambda t: np.imag(integrand(t))

                integral_real, abserr_real = scipy.integrate.quad(integrand_real, 0, 1, epsabs=absTol, epsrel=relTol)
                integral_imag, abserr_imag = scipy.integrate.quad(integrand_imag, 0, 1, epsabs=absTol, epsrel=relTol)
                integral = integral_real + 1j*integral_imag
            else:
                raise ValueError("intMethod must be either 'romb' or 'quad'")

            if np.isnan(integral):
                raise RuntimeError('The integral along the segment %s is NaN.\
                    \nThis is most likely due to a root being on or very close to the path of integration.'%self)

            self._integralCache[args] = integral

        return integral


class ComplexLine(ComplexPath):
    r"""
    A straight line :math:`z` in the complex plane from a to b
    parameterised by

    ..math::

        z(t) = a + (b-a)t, \quad 0\leq t \leq 1


    Parameters
    ----------
    a : float
    b : float
    """
    def __init__(self, a, b):
        self.a, self.b = a, b
        self.dzdt = lambda t: self.b-self.a
        super(ComplexLine, self).__init__()

    def __str__(self):
        return 'ComplexLine from %.3f+%.3fi to %.3f+%.3fi' % (self.a.real, self.a.imag, self.b.real, self.b.imag)

    def __call__(self, t):
        r"""
        The function :math:`z(t) = a + (b-a)t`.

        Parameters
        ----------
        t : float
            A real number :math:`0\leq t \leq 1`.

        Returns
        -------
        complex
            A point on the line in the complex plane.
        """
        return self.a + t*(self.b-self.a)

    def distance(self, z):
        """
        Distance from the point z to the closest point on the line.

        Parameters
        ----------
        z : complex

        Returns
        -------
        float
            The distance from z to the point on the line which is
            closest to z.
        """
        # convert complex numbers to vectors
        A = np.array([self.a.real, self.a.imag])
        B = np.array([self.b.real, self.b.imag])
        Z = np.array([z.real, z.imag])

        # the projection of the point z onto the line a -> b is where
        # the parameter t is
        t = (Z-A).dot(B-A)/abs((B-A).dot(B-A))

        # but the line segment only has 0 <= t <= 1
        t = t.clip(0,1)

        # so the point on the line segment closest to z is
        c = self(t)
        return abs(c-z)

class ComplexArc(ComplexPath):
    r"""
    A circular arc :math:`z` with center z0, radius R, initial angle t0
    and change of angle dt.  The arc is parameterised by

    ..math::

        z(t) = R e^{i(t0 + t dt)} + z0, \quad 0\leq t \leq 1

    Parameters
    ----------
    z0 : complex
    R : float
    t0 : float
    dt : float
    """
    def __init__(self, z0, R, t0, dt):
        self.z0, self.R, self.t0, self.dt = z0, R, t0, dt
        self.dzdt = lambda t: 1j*self.dt*self.R*exp(1j*(self.t0 + t*self.dt))
        super(ComplexArc, self).__init__()

    def __str__(self):
        return 'ComplexArc: z0=%.3f, R=%.3f, t0=%.3f, dt=%.3f' % (self.z0, self.R, self.t0, self.dt)

    def __call__(self, t):
        r"""
        The function :math:`z(t) = R e^{i(t_0 + t dt)} + z_0`.

        Parameters
        ----------
        t : float
            A real number :math:`0\leq t \leq 1`.

        Returns
        -------
        complex
            A point on the arc in the complex plane.
        """
        return self.R*exp(1j*(self.t0 + t*self.dt)) + self.z0

    def distance(self, z):
        """
        Distance from the point z to the closest point on the arc.

        Parameters
        ----------
        z : complex

        Returns
        -------
        float
            The distance from z to the point on the arc which is closest
            to z.
        """
        theta = np.angle(z-self.z0)                 # np.angle maps to (-pi,pi]
        theta = (theta-self.t0)%(2*pi) + self.t0    # put theta in [t0,t0+2pi)

        if ((self.dt > 0 and self.t0 < theta < self.t0+self.dt)
            or (self.dt < 0 and self.t0+self.dt < theta - 2*pi < self.t0)):
            # the closest point to z lies on the arc
            return abs(self.R*exp(1j*theta) + self.z0 - z)
        else:
            # the closest point to z is one of the endpoints
            return min(abs(self(0)-z), abs(self(1)-z))

